@@ -262,7 +262,7 @@ cssp_encode_tspasswordcreds(char *username, char *password, char *domain)
 	memset(&message, 0, sizeof(message));
 
 	// domainName [0]
-	s_realloc(&tmp, strlen(domain) * sizeof(uint16));
+	s_realloc(&tmp, 4 + strlen(domain) * sizeof(uint16));
 	s_reset(&tmp);
 	rdp_out_unistr(&tmp, domain, strlen(domain) * sizeof(uint16));
 	s_mark_end(&tmp);
@@ -275,7 +275,7 @@ cssp_encode_tspasswordcreds(char *username, char *password, char *domain)
 	s_free(h1);
 
 	// userName [1]
-	s_realloc(&tmp, strlen(username) * sizeof(uint16));
+	s_realloc(&tmp, 4 + strlen(username) * sizeof(uint16));
 	s_reset(&tmp);
 	rdp_out_unistr(&tmp, username, strlen(username) * sizeof(uint16));
 	s_mark_end(&tmp);
@@ -289,7 +289,7 @@ cssp_encode_tspasswordcreds(char *username, char *password, char *domain)
 	s_free(h1);
 
 	// password [2]
-	s_realloc(&tmp, strlen(password) * sizeof(uint16));
+	s_realloc(&tmp, 4 + strlen(password) * sizeof(uint16));
 	s_reset(&tmp);
 	rdp_out_unistr(&tmp, password, strlen(password) * sizeof(uint16));
 	s_mark_end(&tmp);
@@ -339,7 +339,7 @@ cssp_encode_tscspdatadetail(unsigned char keyspec, char *card, char *reader, cha
 	// cardName [1]
 	if (card)
 	{
-		s_realloc(&tmp, strlen(card) * sizeof(uint16));
+		s_realloc(&tmp, 4 + strlen(card) * sizeof(uint16));
 		s_reset(&tmp);
 		rdp_out_unistr(&tmp, card, strlen(card) * sizeof(uint16));
 		s_mark_end(&tmp);
@@ -355,7 +355,7 @@ cssp_encode_tscspdatadetail(unsigned char keyspec, char *card, char *reader, cha
 	// readerName [2]
 	if (reader)
 	{
-		s_realloc(&tmp, strlen(reader) * sizeof(uint16));
+		s_realloc(&tmp, 4 + strlen(reader) * sizeof(uint16));
 		s_reset(&tmp);
 		rdp_out_unistr(&tmp, reader, strlen(reader) * sizeof(uint16));
 		s_mark_end(&tmp);
@@ -371,7 +371,7 @@ cssp_encode_tscspdatadetail(unsigned char keyspec, char *card, char *reader, cha
 	// containerName [3]
 	if (container)
 	{
-		s_realloc(&tmp, strlen(container) * sizeof(uint16));
+		s_realloc(&tmp, 4 + strlen(container) * sizeof(uint16));
 		s_reset(&tmp);
 		rdp_out_unistr(&tmp, container, strlen(container) * sizeof(uint16));
 		s_mark_end(&tmp);
@@ -387,7 +387,7 @@ cssp_encode_tscspdatadetail(unsigned char keyspec, char *card, char *reader, cha
 	// cspName [4]
 	if (csp)
 	{
-		s_realloc(&tmp, strlen(csp) * sizeof(uint16));
+		s_realloc(&tmp, 4 + strlen(csp) * sizeof(uint16));
 		s_reset(&tmp);
 		rdp_out_unistr(&tmp, csp, strlen(csp) * sizeof(uint16));
 		s_mark_end(&tmp);
@@ -648,6 +648,7 @@ cssp_read_tsrequest(STREAM token, STREAM pubkey)
 	STREAM s;
 	int length;
 	int tagval;
+	struct stream packet;
 
 	s = tcp_recv(NULL, 4);
 
@@ -673,6 +674,7 @@ cssp_read_tsrequest(STREAM token, STREAM pubkey)
 
 	// receive the remainings of message
 	s = tcp_recv(s, length);
+	packet = *s;
 
 #if WITH_DEBUG_CREDSSP
 	streamsave(s, "tsrequest_in.raw");
@@ -689,6 +691,12 @@ cssp_read_tsrequest(STREAM token, STREAM pubkey)
 	if (!ber_in_header(s, &tagval, &length) ||
 	    tagval != (BER_TAG_CTXT_SPECIFIC | BER_TAG_CONSTRUCTED | 0))
 		return False;
+
+	if (!s_check_rem(s, length))
+	{
+		 rdp_protocol_error("cssp_read_tsrequest(), consume of version from stream would overrun",
+				    &packet);
+	}
 	in_uint8s(s, length);
 
 	// negoToken [1]
@@ -710,7 +718,14 @@ cssp_read_tsrequest(STREAM token, STREAM pubkey)
 		if (!ber_in_header(s, &tagval, &length) || tagval != BER_TAG_OCTET_STRING)
 			return False;
 
-		token->end = token->p = token->data;
+		if (!s_check_rem(s, length))
+		{
+			rdp_protocol_error("cssp_read_tsrequest(), consume of token from stream would overrun",
+					   &packet);
+		}
+
+		s_realloc(token, length);
+		s_reset(token);
 		out_uint8p(token, s->p, length);
 		s_mark_end(token);
 	}
