@@ -3,7 +3,7 @@
    Protocol services - RDP layer
    Copyright (C) Matthew Chapman <matthewc.unsw.edu.au> 1999-2008
    Copyright 2003-2011 Peter Astrand <astrand@cendio.se> for Cendio AB
-   Copyright 2011-2014 Henrik Andersson <hean01@cendio.se> for Cendio AB
+   Copyright 2011-2018 Henrik Andersson <hean01@cendio.se> for Cendio AB
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -270,6 +270,20 @@ rdp_in_unistr(STREAM s, int in_len, char **string, uint32 * str_size)
 	*string = xmalloc(in_len * 2);
 	*str_size = in_len * 2;
 
+	struct stream packet = *s;
+
+	if ((in_len < 0) || ((uint32)in_len >= (RD_UINT32_MAX / 2)))
+	{
+		error("rdp_in_unistr(), length of unicode data is out of bounds.");
+		abort();
+	}
+
+	if (!s_check_rem(s, in_len))
+	{
+		rdp_protocol_error("rdp_in_unistr(), consume of unicode data from stream would overrun", &packet);
+	}
+
+
 #ifdef HAVE_ICONV
 	size_t ibl = in_len, obl = *str_size - 1;
 	char *pin = (char *) s->p, *pout = *string;
@@ -1084,6 +1098,7 @@ process_demand_active(STREAM s)
 {
 	uint8 type;
 	uint16 len_src_descriptor, len_combined_caps;
+	struct stream packet = *s;
 
 	/* at this point we need to ensure that we have ui created */
 	rd_create_ui();
@@ -1091,6 +1106,11 @@ process_demand_active(STREAM s)
 	in_uint32_le(s, g_rdp_shareid);
 	in_uint16_le(s, len_src_descriptor);
 	in_uint16_le(s, len_combined_caps);
+
+	if (!s_check_rem(s, len_src_descriptor))
+	{
+		rdp_protocol_error("rdp_demand_active(), consume of source descriptor from stream would overrun", &packet);
+	}
 	in_uint8s(s, len_src_descriptor);
 
 	DEBUG(("DEMAND_ACTIVE(id=0x%x)\n", g_rdp_shareid));
@@ -1238,74 +1258,122 @@ process_pointer_pdu(STREAM s)
 	}
 }
 
-/* Process bitmap updates */
+/* Process TS_BITMAP_DATA */
+static void
+process_bitmap_data(STREAM s)
+{
+	uint16 left, top, right, bottom, width, height;
+	uint16 cx, cy, bpp, Bpp, flags, bufsize, size;
+	uint8 *data, *bmpdata;
+
+	struct stream packet = *s;
+
+	in_uint16_le(s, left);	/* destLeft */
+	in_uint16_le(s, top);	/* destTop */
+	in_uint16_le(s, right);	/* destRight */
+	in_uint16_le(s, bottom);	/* destBottom */
+	in_uint16_le(s, width);	/* width */
+	in_uint16_le(s, height);	/* height */
+	in_uint16_le(s, bpp);	/* bitsPerPixel */
+	Bpp = (bpp + 7) / 8;
+	in_uint16_le(s, flags);	/* flags */
+	in_uint16_le(s, bufsize);	/* bitmapLength */
+
+	cx = right - left + 1;
+	cy = bottom - top + 1;
+
+	/* FIXME: There are a assumtion that we do not consider in
+	   this code. The value of bpp is not passed to
+	   ui_paint_bitmap() which relies on g_server_bpp for drawing
+	   the bitmap data.
+
+	   Does this means that we can sanity check bpp with g_server_bpp ?
+	 */
+
+
+	if (Bpp == 0 || width == 0 || height == 0)
+	{
+		warning("%s(), [%d,%d,%d,%d], [%d,%d], bpp=%d, flags=%x", __func__,
+			left, top, right, bottom, width, height, bpp, flags);
+		rdp_protocol_error
+			("TS_BITMAP_DATA, unsafe size of bitmap data received from server",
+			 &packet);
+	}
+
+	if ((RD_UINT32_MAX / Bpp) <= (width * height))
+	{
+		warning("%s(), [%d,%d,%d,%d], [%d,%d], bpp=%d, flags=%x", __func__,
+			left, top, right, bottom, width, height, bpp, flags);
+		rdp_protocol_error
+			("TS_BITMAP_DATA, unsafe size of bitmap data received from server",
+			 &packet);
+	}
+
+
+#if DEBUG
+	printf("%s(), [%d,%d,%d,%d], [%d,%d], bpp=%d, flags=%x", __func__,
+	       left, top, right, bottom, width, height, bpp, flags);
+#endif
+	if (flags == 0)
+	{
+		/* read uncompresssed bitmap data */
+		int y;
+		bmpdata = (uint8 *) xmalloc(width * height * Bpp);
+		for (y = 0; y < height; y++)
+		{
+			in_uint8a(s, &bmpdata[(height - y - 1) * (width * Bpp)], width * Bpp);
+		}
+		ui_paint_bitmap(left, top, cx, cy, width, height, bmpdata);
+		xfree(bmpdata);
+		return;
+	}
+
+	if (flags & NO_BITMAP_COMPRESSION_HDR)
+	{
+		size = bufsize;
+	}
+	else
+	{
+		/* Read TS_CD_HEADER */
+		in_uint8s(s, 2);	/* skip cbCompFirstRowSize (must be 0x0000) */
+		in_uint16_le(s, size);	/* cbCompMainBodySize */
+		in_uint8s(s, 2);	/* skip cbScanWidth */
+		in_uint8s(s, 2);	/* skip cbUncompressedSize */
+	}
+
+	/* read compressed bitmap data */
+	if (!s_check_rem(s, size))
+	{
+		rdp_protocol_error("process_bitmap_data(), consume of bitmap data from stream would overrun", &packet);
+	}
+	in_uint8p(s, data, size);
+	bmpdata = (uint8 *) xmalloc(width * height * Bpp);
+	if (bitmap_decompress(bmpdata, width, height, data, size, Bpp))
+	{
+		ui_paint_bitmap(left, top, cx, cy, width, height, bmpdata);
+	}
+	else
+	{
+		warning("%s(), failed to decompress bitmap", __func__);
+	}
+
+	xfree(bmpdata);
+}
+
+
+
+/* Process TS_UPDATE_BITMAP_DATA */
 void
 process_bitmap_updates(STREAM s)
 {
 	uint16 num_updates;
-	uint16 left, top, right, bottom, width, height;
-	uint16 cx, cy, bpp, Bpp, compress, bufsize, size;
-	uint8 *data, *bmpdata;
 	int i;
 
-	in_uint16_le(s, num_updates);
+	in_uint16_le(s, num_updates);	/* rectangles */
 
 	for (i = 0; i < num_updates; i++)
 	{
-		in_uint16_le(s, left);
-		in_uint16_le(s, top);
-		in_uint16_le(s, right);
-		in_uint16_le(s, bottom);
-		in_uint16_le(s, width);
-		in_uint16_le(s, height);
-		in_uint16_le(s, bpp);
-		Bpp = (bpp + 7) / 8;
-		in_uint16_le(s, compress);
-		in_uint16_le(s, bufsize);
-
-		cx = right - left + 1;
-		cy = bottom - top + 1;
-
-		DEBUG(("BITMAP_UPDATE(l=%d,t=%d,r=%d,b=%d,w=%d,h=%d,Bpp=%d,cmp=%d)\n",
-		       left, top, right, bottom, width, height, Bpp, compress));
-
-		if (!compress)
-		{
-			int y;
-			bmpdata = (uint8 *) xmalloc(width * height * Bpp);
-			for (y = 0; y < height; y++)
-			{
-				in_uint8a(s, &bmpdata[(height - y - 1) * (width * Bpp)],
-					  width * Bpp);
-			}
-			ui_paint_bitmap(left, top, cx, cy, width, height, bmpdata);
-			xfree(bmpdata);
-			continue;
-		}
-
-
-		if (compress & 0x400)
-		{
-			size = bufsize;
-		}
-		else
-		{
-			in_uint8s(s, 2);	/* pad */
-			in_uint16_le(s, size);
-			in_uint8s(s, 4);	/* line_size, final_size */
-		}
-		in_uint8p(s, data, size);
-		bmpdata = (uint8 *) xmalloc(width * height * Bpp);
-		if (bitmap_decompress(bmpdata, width, height, data, size, Bpp))
-		{
-			ui_paint_bitmap(left, top, cx, cy, width, height, bmpdata);
-		}
-		else
-		{
-			DEBUG_RDP5(("Failed to decompress data\n"));
-		}
-
-		xfree(bmpdata);
+		process_bitmap_data(s);
 	}
 }
 
@@ -1774,3 +1842,21 @@ rdp_disconnect(void)
 {
 	sec_disconnect();
 }
+
+/* Abort rdesktop upon protocol error
+
+   A protocol error is defined as:
+
+    - A value is outside specified range for example;
+      bpp for a bitmap is not allowed to be greater than the
+      value 32 but is represented by a byte in protocol.
+
+*/
+void
+rdp_protocol_error(const char *message, STREAM s)
+{
+	error("%s(), %s", __func__, message);
+	if (s)
+		hexdump(s->p, s_length(s));
+	exit(0);
+}
